function [Sys,Eval]=FormMatrix(k,a,centers,L,Nsph, c_jl,dl)
% [Sys,Eval] = FormMatrix(k,centers,L,Nsph, c_jl,dl)
% Forms the deterministic system matrix Sys involved in the system of
% equations and forms the matrix Eval used in evaluating the potential
% input: k: wave number
%        a: vector of spheres' radii
%        centers: sphere centers
%        L: spatial truncation number
%        Nsph: number of spheres
%        c_jl: normalizing coefficient
%        dl: normalizing coefficient
% output: Sys: multi-sphere deterministic system matrix
%         Eval: matrix used for evaluation of potential
% Adapted from Nail Gumerov (2001) original code in Fortran

% Recurrence coefficients a_nm,b_nm used in computing the translation
% coefficients
N2=2*L+2;       % total # elements needed for the whole recurrence
anm=zeros(1,(N2+1)^2); bnm=zeros(1,(N2+1)^2);
t=0;
for n=0:N2
    dena = 1/((2*n+1)*(2*n+3));
    denb = 1/((2*n+1)*(2*n-1));
    for m=-n:n
        t=t+1;
        anm(t)=sqrt((n+1+m)*(n+1-m)*dena);
        bnm(t)=sqrt((n-m-1)*(n-m)*denb);
        if m<0, bnm(t)=-bnm(t); end
    end
end
% spherical harmonics required to compute the Eval matrix
jl = sph_bessel(k,a,L);
hl = sph_hankel(k,a,L);
% initialize main matrices
N1=(L+1)^2;     % size of submatrix
Sys=zeros(Nsph*N1);
Eval=zeros(Nsph*N1);
for p=1:Nsph
    for q=p:Nsph
        if q==p      % diagonal blocks
            t=0;
            for el=0:L
                for m=-el:el
                    t=t+1;
                    Sys((q-1)*N1+t,(q-1)*N1+t)=dl(q,el+1);
                    Eval((q-1)*N1+t,(q-1)*N1+t)=hl(q,el+1);
                end
            end
        else        % off diagonal blocks
            % compute S-multipoles for pair pq: S(r_pq)
            r_pq=centers(q,:)-centers(p,:);
            [theta,phi,r]=cart2sph(r_pq(1),r_pq(2),r_pq(3));
            myPhi=theta; myTheta=pi/2-phi;      % Nail's convention
            hl_pq=sph_hankel(k,r,N2);           % needed for Snmpq
            Snmpq=zeros(1,(N2+1)^2);
            t=1; exx=exp(i*myPhi);
            for n=0:N2
                Y = sqrt((2*n+1)/(2*4*pi)) * legendre(n,cos(myTheta),'sch');
                Y(1) = sqrt(2)*Y(1);     % normalization const different for order 0
                ex=1;
                for m=0:n
                    Snmpq(t+m)=hl_pq(n+1)*Y(m+1)*ex;
                    Snmpq(t-m)=hl_pq(n+1)*Y(m+1)*ex';   % conjugate
                    ex=ex*exx;
                end
                t=t+2*n+2;
            end
            
            % compute SR-multipoles for pair pq: (S|R)(r_pq)
            tau=SRcoef(L,anm,bnm,Snmpq);
            % fill the system matrix blocks qp and pq
            sign1=-1;
            for el=0:L
                sign1=-sign1;
                Sys((q-1)*N1+(el^2+1:(el+1)^2) , (p-1)*N1+1:p*N1)= ...
                    c_jl(q,el+1) * tau(el^2+1:(el+1)^2 , 1:N1);
                Eval((q-1)*N1+(el^2+1:(el+1)^2) , (p-1)*N1+1:p*N1)= ...
                    jl(q,el+1) * tau(el^2+1:(el+1)^2 , 1:N1);
                sign2=-1;
                for n=0:L
                    sign2=-sign2;
                    Sys((p-1)*N1+(el^2+1:(el+1)^2) , (q-1)*N1+(n^2+1:(n+1)^2))= ...
                        sign1*sign2*c_jl(p,el+1) * tau(el^2+1:(el+1)^2 , n^2+1:(n+1)^2);
                    Eval((p-1)*N1+(el^2+1:(el+1)^2) , (q-1)*N1+(n^2+1:(n+1)^2))= ...
                        sign1*sign2*jl(p,el+1) * tau(el^2+1:(el+1)^2 , n^2+1:(n+1)^2);
                end
            end
            
        end         % q~=p off diagonal blocks
    end     % q-loop
end     % p-loop