function [var,mean]=compMomentsF(A,CvecIn,meanXiEta,lambda,L,Nw,Nsph,ngrid,k,a,ess,centers,M)
% [var,mean]=compMoments(A,L,Nw,Nsph,ngrid,k,a,ess,centers,M)
% Computes mean and variance of the discrete solution A
% input: A: matrix of coefficients of expansion of solution
%        L: spatial trubcation number
%        Nw: stochastic truncation number
%        Nsph: number of spheres
%        ngrid: no of subdivisions in the surface mesh in each coordinate
%        k: wave number
%        a: radius
%        ess: incidence direction vector
%        centers: spheres centers
%        M: truncation number of KL expansion
%        CvecIn: mean and eigenvectors of covariance matrix of incident field
%        lambda: eigenvalues of incident field
%        meanXiEta: covariance <eta_t xi_r>, t=1,...,Nw, r=1,...,M
% output: mean: mean of solution at each mesh point, for every sphere
%         var: variance of solution at each mesh point, for every sphere

% mesh defining the spheres' surfaces: theta*phi (outer product)
nphi=ngrid-1; ntheta=ceil(ngrid/2)-1;
mean=zeros(ntheta+1,nphi+1,Nsph); var=zeros(ntheta+1,nphi+1,Nsph); % total field
meanIn=zeros(ntheta+1,nphi+1,Nsph); varIn=zeros(ntheta+1,nphi+1,Nsph); % incident field
CovInS=zeros(ntheta+1,nphi+1,Nsph);     % covariance between incident and scattered fields
theta = pi*[0:ntheta]'/ntheta;
phi = 2*pi*[0:nphi]/nphi;
L2=(L+1)^2;

% prepare spherical harmonics' values at mesh points
Ylm=[];         % Legendre part
for el=0:L
    Ytemp = sqrt((2*el+1)/(2*4*pi)) * legendre(el,cos(theta'),'sch');
	Ytemp(1,:) = sqrt(2)*Ytemp(1,:);     % normalization const different for order 0
	Ylm = [Ylm; Ytemp];
end
clear Ytemp
expon=zeros(2*L+1,nphi+1);     % exponential (imaginary) part
ex=ones(1,nphi+1);
exx=exp(i*phi);
for m=0:L
    expon(L+1+m,:)=ex;
    expon(L+1-m,:)=conj(ex);
    ex=ex.*exx;
end
clear ex exx

%==============   Moments of incident field   ===========================
% Mean of incident field
s=1;        % index inside CvecIn(:,1)
yidx=1;     % 1st row index of Ylm in current "el-block"
for el=0:L
    for m=-el:el
        % spherical harmonic evaluated at all mesh points
        Y=Ylm(yidx+abs(m),:).'*expon(L+1+m,:);
        for q=1:Nsph        % same harmonic for each sphere
            meanIn(:,:,q) = meanIn(:,:,q) + CvecIn((q-1)*L2+s,1) * Y;
        end
        s=s+1;
    end
    yidx=yidx+el+1;
end
% Variance of incident field
for nyu=1:M
	s=1;        % index inside columns of allvec
	yidx=1;     % 1st row index of Ylm in current "el-block"
    sumup=zeros(ntheta+1,nphi+1,Nsph);       % partial temporary sum
	for el=0:L
        for m=-el:el
            % spherical harmonic evaluated at all mesh points
            Y=Ylm(yidx+abs(m),:).'*expon(L+1+m,:);
            for q=1:Nsph        % same harmonic for each sphere
                sumup(:,:,q) = sumup(:,:,q) + CvecIn((q-1)*L2+s,nyu+1) * Y;
            end
            s=s+1;
        end
        yidx=yidx+el+1;
	end
    varIn = varIn + lambda(nyu)*sumup.*conj(sumup);
end

%===========   Covariance between incident and scattered fields   ========
for nyu=1:M
    XiEtaVec=meanXiEta(:,nyu)';
	s=1;        % index inside columns of allvec
	yidx=1;     % 1st row index of Ylm in current "el-block"
    sumup=zeros(ntheta+1,nphi+1,Nsph);   % temp sum for scattered field
    sumup2=zeros(ntheta+1,nphi+1,Nsph);  % temp sum for incident field
	for el=0:L
        for m=-el:el
            % spherical harmonic evaluated at all mesh points
            Y=Ylm(yidx+abs(m),:).'*expon(L+1+m,:);
            for q=1:Nsph        % same harmonic for each sphere
                sumup(:,:,q) = sumup(:,:,q) + sum(A((q-1)*L2+s,:).*XiEtaVec) * Y;
                sumup2(:,:,q) = sumup2(:,:,q) + CvecIn((q-1)*L2+s,nyu+1) * Y;
            end
            s=s+1;
        end
        yidx=yidx+el+1;
	end
    CovInS = CovInS + sqrt(lambda(nyu))*conj(sumup2).*(sumup);
end     % for-nyu
clear sumup sumup2
%==============   Moments of scattered field   ===========================
% compute scattered field's covariance matrix
sMean=zeros(Nsph*L2,1);     % sample mean vector of S-multipole coefficients
sCov=zeros(Nsph*L2);        % covariance matrix of S-multipole coefficients
for tt=1:Nw
    Anm=A(:,tt);
    sMean=sMean+Anm;
    sCov=sCov + Anm*Anm';   % outer (Kronecker) product (conjugate transpose)
end
sMean=sMean/Nw;
sCov = sCov/Nw - sMean*sMean';
% spectral decomposition of covariance matrix of scattered field
% M=4;      % the same global M. Directly affected by M for incident field
% options: symmetric, non-real matrix. Please no diagnostic display!
opts.issym=1; opts.isreal=0; opts.disp=0;
[allvec,lambdaS] = eigs(sCov,M,'lm',opts);    % eigendecomposition of covariance matrix
lambdaS=diag(lambdaS);               % convert eigenavalues' matrix to a vector
clear sCov

% Mean of scattered field
s=1;        % index inside sMean
yidx=1;     % 1st row index of Ylm in current "el-block"
for el=0:L
    for m=-el:el
        % spherical harmonic evaluated at all mesh points
        Y=Ylm(yidx+abs(m),:).'*expon(L+1+m,:);
        for q=1:Nsph        % same harmonic for each sphere
            mean(:,:,q) = mean(:,:,q) + sMean((q-1)*L2+s) * Y;
        end
        s=s+1;
    end
    yidx=yidx+el+1;
end
% mean of total field
%mean=abs(mean+meanIn);
mean=abs(mean);

% Variance of scattered field
for nyu=1:M
	s=1;        % index inside columns of allvec
	yidx=1;     % 1st row index of Ylm in current "el-block"
    sumup=zeros(ntheta+1,nphi+1,Nsph);       % partial temporary sum
	for el=0:L
        for m=-el:el
            % spherical harmonic evaluated at all mesh points
            Y=Ylm(yidx+abs(m),:).'*expon(L+1+m,:);
            for q=1:Nsph        % same harmonic for each sphere
                sumup(:,:,q) = sumup(:,:,q) + allvec((q-1)*L2+s,nyu) * Y;
            end
            s=s+1;
        end
        yidx=yidx+el+1;
	end
    var = var + lambdaS(nyu)*sumup.*conj(sumup);
end
% variance of total field
%var=abs(var+varIn+2*CovInS);
%var=abs(var+varIn);
var=abs(var);