% Driver for the stochastic multi-Helmholtz problem with multipole
% reexpansion

% ----------- Constant Parameters ---------------------------
% Sphere parameters
k=3;               % wave number
sigma=[0,0];            % surface admittance
a=[1,1.3253];                % spheres radii
centers=[[0,0,0];[0,-2.3253,0]];      % spheres centers: Nsph x 3 matrix
% Spatial parameters
s=[0,0];      % direction of incidence for plane wave (unused): theta_in, phi_in
% origin of monopole source incident wave built in IncidWav.m
Nsph=2;             % number of spheres
% Stochastic parameters
alpha=sqrt(3);      % uniform distribution of random variables
                    % over interval [-alpha,alpha]
% Truncation numbers
L=10;               % outer sum in spatial expansion
M=3;                % no. of terms in Karhunen-Loeve expansion (M=0 for
                    % deterministic)
nw=4;               % no. of subdivisions in each stochastic coordinate direction
Nw=nw^M;            % stochastic summation truncation number

% ------------ Compute left hand side elements --------------
% Diagonal elements of the D-matrices
dl = compD(k,a,sigma,L);      % N*(L+1) matrix. dl(:,el+1)=d_el
% Diagonal elements g_t of the Grammian matrix are all equal
g=1/Nw;
% ------------ Compute right hand side elements -------------
meanEta=1/Nw;                           % <eta_t>,  t=1,...,Nw, are all equal
meanXiEta = genMeanXiEta(M,nw,alpha);   % <xi_r eta_t>, t=1,...,Nw, r=1,...,M.
% Coefficients appearing in the c(r) vectors
c_jl = compJ(k,a,sigma,L);              % N*(L+1) matrix
% Coefficients of expansion of the mean of the incident wave Psi_in(0)
            disp('coeff Psi_0: A long one!')
c0 = coeffPsi0(a,centers,s,k,L,Nsph,c_jl);           % (L+1)^2 x Nsph vector
% Eigenproblem: compute all eigenvalues then choose M largest.
all_lambda = compLambda(M);             % M+1 vector
% Save elements needed to compute the sums in the rhs in a special data
% structure
RHS = compRHS(k,a,L,M, c_jl,all_lambda);

% ------------ Solve for the discrete solution --------------
% Sum up right hand sides and solve for A_lmt
% Matrix A holds the vectors a_t as columns rather than as one long vector
            disp('Forming system matrix...')
A=zeros(Nsph*(L+1)^2,Nw);
% Sys is the deterministic system matrix
% Eval is used to evaluate the potential from the discrete solution
[Sys,Eval]=FormMatrix(k,a,centers,L,Nsph, c_jl,dl);
if Nsph>1
    [EL,U,P]=lu(g*Sys);
else
    diagSys=diag(Sys); diagEval=diag(Eval);     % they are diagonal
    clear Eval
end
clear Sys
for t=1:Nw                  % stochastic index
    rsum=meanEta*c0(:);     % right hand side sum: Nsph*(L+1)^2 vector
    for r=1:M
        c_r=zeros(Nsph*(L+1)^2,1); % vector of coefficients of expansion of eigenfunction
        c_r((0:Nsph-1)*(L+1)^2+RHS(2,r))=RHS(3:end,r); %  in a spherical harmonics basis
        rsum = rsum + sqrt(RHS(1,r))*meanXiEta(t,r)*c_r;
    end
    % compute values of A_lmt for all spheres
    if Nsph>1
        A(:,t) = -U\(EL\(P*rsum));        % -(1/g) * U\(EL\(P*rsum));
    else
        A(:,t) = -(1/g) * rsum./diagSys;
    end
end
if Nsph>1, clear EL U P, else clear diagSys, end

% ------- Compute and plot statistical properties of the solution -------
ngrid=100;           % no. of subdivisions along theta and phi for sphere surface
            disp('compute moments: A longer one!')
% prepare the coefficients to compute the moments
if Nsph>1
    for t=1:Nw, A(:,t)=Eval*A(:,t); end
    clear Eval
else
    for t=1:Nw, A(:,t)=diagEval.*A(:,t); end
    clear diagEval
end
% compute mean and variance of solution A
[var,mean]=compMoments(A,L,Nw,Nsph,ngrid,k,a,s,centers);
% plot solution implicitly in 3D
plotParam3D(a,centers,ngrid,Nsph, mean,var);