% Driver for the stochastic multi-Helmholtz problem with multipole
% reexpansion

% ----------- Constant Parameters ---------------------------
% Sphere parameters
 k=3;               % wave number
sigma=[0,0];            % surface admittance
a=[1,1.3253];                % spheres radii
centers=[[0,0,0];[0,-2.3253,0]];      % spheres centers: Nsph x 3 matrix
% Spatial parameters
s=[0,0];      % direction of incidence for plane wave (unused): theta_in, phi_in
% origin of monopole source incident wave built in IncidWav.m
Nsph=2;             % number of spheres
% Stochastic parameters
alpha=sqrt(3);      % uniform distribution of random variables
                    % over interval [-alpha,alpha]
 varGauss=0.5;     % source location variance (updated from runner)                    
% Truncation numbers
L=10;               % outer sum in spatial expansion
%L=max(5,floor(3.1604*k));      % computed automatically based on this problem
M=4;                % no. of terms in Karhunen-Loeve expansion (M=0 for
                    % deterministic)
nw=4;               % no. of subdivisions in each stochastic coordinate direction
Nw=nw^M;            % stochastic summation truncation number

% ------------ Compute left hand side elements --------------
% Diagonal elements of the D-matrices
dl = compD(k,a,sigma,L);      % N*(L+1) matrix. dl(:,el+1)=d_el
% Diagonal elements g_t of the Grammian matrix are all equal
g=1/Nw;
% ------------ Compute right hand side elements -------------
meanEta=1/Nw;                           % <eta_t>,  t=1,...,Nw, are all equal
meanXiEta = genMeanXiEta(M,nw,alpha);   % <xi_r eta_t>, t=1,...,Nw, r=1,...,M.
% Coefficients appearing in the c(r) vectors
c_jl = compJ(k,a,sigma,L);              % N*(L+1) matrix
%******************************************
% Eigenproblem: compute M largest eigenvalues and corresponding eigenvectors
[lambda,Cvec,CvecIn]=genEigsMult(k,a,centers,L,M,Nsph, c_jl,varGauss);
% ------------ Solve for the discrete solution --------------
% Sum up right hand sides and solve for A_lmt
% Matrix A holds the vectors a_t as columns rather than as one long vector
            disp('Forming and solving system matrix...'), tic
A=zeros(Nsph*(L+1)^2,Nw);
% Sys is the deterministic system matrix
% Eval is used to evaluate the potential from the discrete solution
[Sys,Eval]=FormMatrix(k,a,centers,L,Nsph, c_jl,dl);
if Nsph>1
    [EL,U,P]=lu(Sys);
else
    diagSys=diag(Sys); diagEval=diag(Eval);     % they are diagonal
    clear Eval
end
clear Sys
% solution of spatial part
for r=0:M
    if Nsph>1
        CvecSolve(:,r+1) = -U\(EL\(P*Cvec(:,r+1)));
    else
        CvecSolve(:,r+1) = -Cvec(:,r+1)./diagSys;
    end
end
if Nsph>1, clear EL U P, else clear diagSys, end
% Kronecker product with stochastic part
A=repmat(CvecSolve(:,1),1,Nw);
for r=1:M
    sqrtlN=sqrt(lambda(r))*Nw;
    for t=1:Nw
        A(:,t) = A(:,t) + sqrtlN*meanXiEta(t,r)*CvecSolve(:,r+1);
    end
end
%clear Cvec
% ------- Compute and plot statistical properties of the solution -------
ngrid=30;           % no. of subdivisions along theta and phi for sphere surface
          toc,  disp('compute moments: A longer one!'), tic
% prepare the coefficients to compute the moments
if Nsph>1
    for t=1:Nw, A(:,t)=Eval*A(:,t); end
    clear Eval
else
    for t=1:Nw, A(:,t)=diagEval.*A(:,t); end
    clear diagEval
end
% compute mean and variance of solution A
[var,mean]=compMomentsF(A,CvecIn,meanXiEta,lambda,L,Nw,Nsph,ngrid,k,a,s,centers,M);
%[varS,meanS]=compMoments(A,L,Nw,Nsph,ngrid,k,a,s,centers);
            toc
% plot solution implicitly in 3D
plotParam3D(a,centers,ngrid,Nsph, mean,var);