function [lambda,Cvec,CvecIn]=genEigsMult(k,a,centers,L,M,Nsph, c_jl,varGauss)
% [lambda,Cvec]=genEigsMult(k,a,centers,L,M,Nsph, c_jl,varGauss)
% input: k: wave number
%        a: sphere radii
%        centers: spheres centers
%        L: spatial truncation number
%        M: Karhunen-Loeve expansion truncation number
%        Nsph: number of spheres
%        c_jl: normalizing coefficient
%        varGauss: variance of normal distribution
% output: lambda: first M largest eigenvalues (for each sphere)
%         Cvec: corresponding eigenvectors and sample mean discretized in a
%               spherical harmonic basis. Used to solve main system
%         CvecIn: eigenvectors used to evaluate incident field
% The function genEigs does the following:
%  - generate a random sample of measurements of the incident field over the 
% spheres' surface.
%  - compute the sample mean and sample covariance matrix.
%  - retain the first M largest eigenvalues and corresponding eigenvectors
%  - expand the sample mean and eigenvectors in a spherical harmonics basis

smpl_size=100;     % size of random sample population
L2=(L+1)^2;
            fprintf('generating sample of %d observations\n',smpl_size); tic
sMean=zeros(Nsph*L2,1);     % sample mean vector of R-multipole coefficients
sCov=zeros(Nsph*L2);        % Galerkin discretized covariance matrix
myuGauss=[0,0,10*a(1)];     % mean of normal distribution
%varGauss=0.01;             % variance of normal distribution (now parameter)
for tt=1:smpl_size
    % 'theta' and 'phi' are uniform and 'r' is normal
    th=pi*rand; ph=2*pi*rand; rr=abs(sqrt(varGauss)*randn);
    rsource=myuGauss + [rr*sin(th)*cos(ph) , rr*sin(th)*sin(ph) , rr*cos(th)];
    Cnm = genSample(a,centers,k,rsource, Nsph,L);  % one sample vector
    sMean=sMean+Cnm;
    sCov=sCov + Cnm*Cnm';   % outer (Kronecker) product (conjugate transpose)
end
sMean=sMean/smpl_size;
sCov = sCov/(smpl_size-1) - smpl_size/(smpl_size-1)*sMean*sMean';
% adjust covariance matrix entries with appropriate coefficients
jl=sph_bessel(k,a,L);       % spherical bessel functions
for el=0:L
    for qq=1:Nsph
        sCov((qq-1)*L2+ (el^2+1 : (el+1)^2) , :)= ...
            sCov((qq-1)*L2+ (el^2+1 : (el+1)^2) , :) * jl(qq,el+1);
    end
end
for el=0:L
    for qq=1:Nsph
        sCov(: , (qq-1)*L2+ (el^2+1 : (el+1)^2))= ...
            sCov(: , (qq-1)*L2+ (el^2+1 : (el+1)^2)) * jl(qq,el+1);
    end
end
            toc
            disp('Eigendecomposition'); tic
% spectral decomposition of covariance matrix. First M largest magnitude
% eigenvalues
% options: symmetric, non-real matrix. Please no diagnostic display!
opts.issym=1; opts.isreal=0; opts.disp=0;
[allvec,lambda] = eigs(sCov,M,'lm',opts);     % eigendecomposition of covariance matrix
lambda=diag(lambda);            % convert eigenavalues' matrix to a vector
            toc
Cvec=zeros(Nsph*L2,M+1);
CvecIn=zeros(Nsph*L2,M+1);
% creating the mean vector c_0 used in solution of system
for el=0:L
    for qq=1:Nsph
        Cvec((qq-1)*L2+ (el^2+1 : (el+1)^2),1)= ...
            sMean((qq-1)*L2+ (el^2+1 : (el+1)^2)) * c_jl(qq,el+1);
    end
end
% creating the mean vector c_0 used in evaluation of incident field
for el=0:L
    for qq=1:Nsph
        CvecIn((qq-1)*L2+ (el^2+1 : (el+1)^2),1)= ...
            sMean((qq-1)*L2+ (el^2+1 : (el+1)^2)) * jl(qq,el+1);
    end
end

% creating the first M eigenvectors mult. by appropriate coefficients
for el=0:L
    for qq=1:Nsph
        Cvec((qq-1)*L2+ (el^2+1 : (el+1)^2),2:M+1)= ...
            allvec((qq-1)*L2+ (el^2+1 : (el+1)^2),1:M) * c_jl(qq,el+1)/jl(qq,el+1);
    end
end
% creating the first M eigenvectors used to evaluate the incident field
CvecIn(:,2:M+1)=allvec;

%***********************  Subfunctions  ****************************%
function Cnm = genSample(a,centers,k,rsource, Nsph,L)
% Cnm = genSample(a,centers,k,rsource, Nsph)
% Generates one sample vector of R-multipole coefficients
% for the source located at 'rsource'

L2=(L+1)^2;         % spherical harmonics discretization truncation number
Q=1;
% create vectors r_sq from spheres' centers to source
rr=zeros(1,Nsph); th=zeros(1,Nsph); ph=zeros(1,Nsph);
for qq=1:Nsph
    r_sq=rsource-centers(qq,:);     % vector from q_th sphere center to source
    [theta,phi,r]=cart2sph(r_sq(1),r_sq(2),r_sq(3));  % cartesian to spherical
    rr(qq)=r; ph(qq)=theta; th(qq)=pi/2-phi;          % Nail's convention
end
hl_sq = sph_hankel(k,rr,L);     % spherical Hankel
Cnm=zeros(Nsph*L2,1);
% create a sample vector of R-multipole coefficients
t=1; exx=exp(i*ph);
for n=0:L
    Y = sqrt((2*n+1)/(2*4*pi)) * legendre(n,cos(th),'sch');
    Y(1,:) = sqrt(2)*Y(1,:);     % normalization const different for order 0
    ex=ones(1,Nsph);
    for m=0:n
        for qq=1:Nsph
            Cnm((qq-1)*L2+ t+m)=hl_sq(qq,n+1)*Y(m+1,qq)*conj(ex(qq));
            Cnm((qq-1)*L2+ t-m)=hl_sq(qq,n+1)*Y(m+1,qq)*ex(qq);
        end
        ex=ex.*exx;
    end
    t=t+2*n+2;
end
Cnm = Q*i*k*Cnm;
%*******************************************************************%